/*
    (C) 1995-97 AROS - The Amiga Research OS
    $Id: fault.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: dos.library function Fault()
    Lang: english
*/
#include <aros/options.h>
#include "dos_intern.h"
#include <proto/exec.h>
#if PassThroughErrnos
#   include <errno.h>
#endif

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH4(LONG, Fault,

/*  SYNOPSIS */
	AROS_LHA(LONG  , code, D1),
	AROS_LHA(STRPTR, header, D2),
	AROS_LHA(STRPTR, buffer, D3),
	AROS_LHA(LONG  , len, D4),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 78, Dos)

/*  FUNCTION
	Fault will obtain the error message string for the given error
	code. First the header string is copied to the buffer, followed
	by a ":" (colon), then the NULL terminated string for the error
	message into the buffer.

	By convention, error messages are ALWAYS less than 80 (plus 1 for
	NULL termination), and ideally less than 60 characters.

	If the error code is not know, then the string "Unknown error"
	followed by the error number will be added to the string.

    INPUTS
	code	-   The error code.
	header	-   The string to prepend to the buffer before the error
		    text. This may be NULL in which case nothing is prepended.
	buffer	-   The destination buffer.
	len	-   Length of the buffer.

    RESULT
	Number of characters placed in the buffer, may be 0.

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    LONG index = 0;
    STRPTR theString;

    DEBUG_ERRORS(dprintf("Fault: code = %ld, buffer = $%lx, size = %ld, header = \"%s\"\n",
			 code, (ULONG) buffer, len, header ? header : (STRPTR) "<NULL>"));

    /* Do this to make sure there is room for a NULL terminator */
    len--;

    if (header)
    {
	while ((index < len) && *header)
	{
	    buffer[index++] = *header++;
	}

	buffer[index++] = ':';
	buffer[index++] = ' ';
    }

    theString = DosGetString(code);
#if PassThroughErrnos
    if ((!theString) && (code & PassThroughErrnos))
    {
	theString = strerror (code ^ PassThroughErrnos);
    }
#endif
    if (theString)
    {
	while ((index < len) && *theString)
	{
	    buffer[index++] = *theString++;
	}
    }
    else
    {
	UBYTE tmp[32];   /* "Error " */
	LONG array[2] = {(LONG) DosGetString(-117), code};

	RawDoFmt("%s%ld", array, (void (*)(void)) 0, tmp);
	index = strlen(tmp);
	if (index > len)
	{
	    index = len;
	}

	memcpy(buffer, tmp, (size_t) index);
    }
    buffer[index] = '\0';

    DEBUG_ERRORS(dprintf("Fault: ret = %ld, buffer = \"%s\"\n", index, buffer));

    return index;

    AROS_LIBFUNC_EXIT
} /* Fault */
